@@ -102,7 +102,7 @@ start_amandates(
 	rc = amroflock(fileno(amdf), amandates_file);
 
     if(rc == -1) {
-	error("could not lock %s: %s", amandates_file, strerror(errno));
+	error(_("could not lock %s: %s"), amandates_file, strerror(errno));
 	/*NOTREACHED*/
     }
 
@@ -137,7 +137,7 @@ start_amandates(
     }
 
     if(ferror(amdf)) {
-	error("reading %s: %s", amandates_file, strerror(errno));
+	error(_("reading %s: %s"), amandates_file, strerror(errno));
 	/*NOTREACHED*/
     }
 
@@ -157,7 +157,7 @@ finish_amandates(void)
 
     if(updated) {
 	if(readonly) {
-	    error("updated amandates after opening readonly");
+	    error(_("updated amandates after opening readonly"));
 	    /*NOTREACHED*/
 	}
 
@@ -166,7 +166,7 @@ finish_amandates(void)
 	    for(level = 0; level < DUMP_LEVELS; level++) {
 		if(amdp->dates[level] == EPOCH) continue;
 		qname = quote_string(amdp->name);
-		fprintf(amdf, "%s %d %ld\n",
+		g_fprintf(amdf, "%s %d %ld\n",
 			qname, level, (long) amdp->dates[level]);
 		amfree(qname);
 	    }
@@ -174,11 +174,11 @@ finish_amandates(void)
     }
 
     if(amfunlock(fileno(amdf), g_amandates_file) == -1) {
-	error("could not unlock %s: %s", g_amandates_file, strerror(errno));
+	error(_("could not unlock %s: %s"), g_amandates_file, strerror(errno));
 	/*NOTREACHED*/
     }
     if (fclose(amdf) == EOF) {
-	error("error [closing %s: %s]", g_amandates_file, strerror(errno));
+	error(_("error [closing %s: %s]"), g_amandates_file, strerror(errno));
 	/*NOTREACHED*/
     }
     amdf = NULL;
@@ -258,8 +258,8 @@ enter_record(
     if(level < 0 || level >= DUMP_LEVELS || dumpdate < amdp->dates[level]) {
 	qname = quote_string(name);
 	/* this is not allowed, but we can ignore it */
-        dbprintf(("amandates botch: %s lev %d: new dumpdate %ld old %ld\n",
-		  qname, level, (long) dumpdate, (long) amdp->dates[level]));
+        dbprintf(_("amandates botch: %s lev %d: new dumpdate %ld old %ld\n"),
+		  qname, level, (long) dumpdate, (long) amdp->dates[level]);
 	amfree(qname);
 	return;
     }
@@ -284,8 +284,8 @@ amandates_updateone(
     if(level < 0 || level >= DUMP_LEVELS || dumpdate < amdp->dates[level]) {
 	/* this is not allowed, but we can ignore it */
 	qname = quote_string(name);
-	dbprintf(("amandates updateone: %s lev %d: new dumpdate %ld old %ld",
-		  name, level, (long) dumpdate, (long) amdp->dates[level]));
+	dbprintf(_("amandates updateone: %s lev %d: new dumpdate %ld old %ld"),
+		  name, level, (long) dumpdate, (long) amdp->dates[level]);
 	amfree(qname);
 	return;
     }
