@@ -39,9 +39,9 @@
 #include "security.h"
 #include "event.h"
 
-#define auth_debug(i,x) do {		\
+#define auth_debug(i, ...) do {		\
 	if ((i) <= debug_auth) {	\
-	    dbprintf(x);		\
+	    dbprintf(__VA_ARGS__);	\
 	}				\
 } while (0)
 
@@ -76,13 +76,15 @@ struct tcp_conn {
     int			refcnt;			/* number of handles using */
     int			handle;			/* last proto handle read */
     void		(*accept_fn)(security_handle_t *, pkt_t *);
-    struct sockaddr_storage	peer;
+    sockaddr_union	peer;
     TAILQ_ENTRY(tcp_conn) tq;			/* queue handle */
     int			(*recv_security_ok)(struct sec_handle *, pkt_t *);
     char *		(*prefix_packet)(void *, pkt_t *);
     int			toclose;
     int			donotclose;
     int			auth;
+    char *              (*conf_fn)(char *, void *);
+    void *              datap;
 #ifdef KRB5_SECURITY
     gss_ctx_id_t	gss_context;
 #endif
@@ -107,7 +109,7 @@ struct sec_handle {
     } fn;
     void *		arg;		/* argument to pass function */
     event_handle_t *	ev_timeout;	/* timeout handle for recv */
-    struct sockaddr_storage	peer;
+    sockaddr_union	peer;
     int			sequence;
     event_id_t		event_id;
     char *		proto_handle;
@@ -163,7 +165,7 @@ extern struct connq_s connq;
 typedef struct udp_handle {
     const struct security_driver *driver;	/* MUST be first */
     dgram_t dgram;		/* datagram to read/write from */
-    struct sockaddr_storage peer;	/* who sent it to us */
+    sockaddr_union peer;	/* who sent it to us */
     pkt_t pkt;			/* parsed form of dgram */
     char *handle;		/* handle from recvd packet */
     int sequence;		/* seq no of packet */
@@ -206,8 +208,11 @@ typedef struct udp_handle {
 
 int	sec_stream_auth(void *);
 int	sec_stream_id(void *);
-void	sec_accept(const security_driver_t *, int, int,
-		    void (*)(security_handle_t *, pkt_t *));
+void	sec_accept(const security_driver_t *,
+		   char *(*)(char *, void *),
+		   int, int,
+		   void (*)(security_handle_t *, pkt_t *),
+		   void *);
 void	sec_close(void *);
 void	sec_connect_callback(void *);
 void	sec_connect_timeout(void *);
@@ -250,7 +255,7 @@ void	udp_recvpkt_cancel(void *);
 void	udp_recvpkt_callback(void *);
 void	udp_recvpkt_timeout(void *);
 int	udp_inithandle(udp_handle_t *, struct sec_handle *, char *hostname,
-		       struct sockaddr_storage *, in_port_t, char *, int);
+		       sockaddr_union *, in_port_t, char *, int);
 void	udp_netfd_read_callback(void *);
 
 struct tcp_conn *sec_tcp_conn_get(const char *, int);
@@ -265,7 +270,7 @@ char *	check_user_ruserok    (const char *host,
 				struct passwd *pwd,
 				const char *user);
 char *	check_user_amandahosts(const char *host,
-			        struct sockaddr_storage *addr,
+			        sockaddr_union *addr,
 				struct passwd *pwd,
 				const char *user,
 				const char *service);
@@ -276,9 +281,5 @@ ssize_t net_read_fillbuf(int, int, void *, size_t);
 void	show_stat_info(char *a, char *b);
 int     check_name_give_sockaddr(const char *hostname, struct sockaddr *addr,
 				 char **errstr);
-int     check_addrinfo_give_name(struct addrinfo *res, const char *hostname,
-				 char **errstr);
-int	try_resolving_hostname(const char *hostname,
-			       char **cannonname);
 
 #endif /* _SECURITY_INFO_H */
